/* -*- mode: c++; c-basic-offset: 3; -*- */
#ifndef FRAMEXMIT_MATCHIF_HH
#define FRAMEXMIT_MATCHIF_HH
#include <vector>
#include <netinet/in.h>
#include <net/if.h>
#include <iosfwd>

namespace framexmit {
   /** interface list class
    */ 
   class if_list {
   public:
      /** Interface description structure
       */  
      struct interface_type {
         char		name[IFNAMSIZ];
         in_addr_t		addr;
         in_addr_t		mask;
         in_addr_t		brdaddr;
      };

      /// Internal interface list structure 
      typedef std::vector<interface_type> interfaceList;
      /// Interface list iterator
      typedef interfaceList::iterator  iflist_iter;
      /// Constant interface list iterator
      typedef interfaceList::const_iterator const_iflist_iter;

      /**  Construct an interface list from a temporary socket
        *  \brief Default constructor
        */
      if_list(void);

     /**  Construct an interface list from an existant socket
       *  \brief Constructor from socket.
       *  \param s Socket fd.
       */
     if_list(int s);

     /**  destroy an interface list
       *  \brief Destructor
       */
     ~if_list(void) = default;
     /**  display the list of devices.
       *  \brief display device list.
       *  \param output stl ostream
       */
     void display(std::ostream& out) const;

     /** Test for empty interface list
       * \brief Test for empty
       * \return true if empty
       */
     bool empty(void) const;

     /**  End of interface list
       */
     const_iflist_iter end(void) const;

     /**  get a list of interfaces.
       */
     bool getInterfaces (int sock); 

     /**  Match an interface to a specified address and return the 
       *  interfaceList iterator to thecorrespoding interface descriptor.
       *  \brief Find the interface matching the specified address
       *  \param addr IP address on network to be matched.
       *  \return iterator pointing to request interface or end()
       */
     const_iflist_iter match(const char* net) const;

  private:
     interfaceList iList;
  };

  inline bool
  if_list::empty(void) const {
     return iList.empty();
  }

  inline if_list::const_iflist_iter
  if_list::end(void) const {
     return iList.end();
  }

  /**  Find the host address on the interface that routes to the address 
    *  specified by \a net.
    *  \brief Find address for network. 
    *  \param sock   Socket number.
    *  \param net    Address on the network to be bound to.
    *  \param i_addr IP address.
    */
  bool matchInterface (int sock, const char* net, in_addr& i_addr);
}

#endif
